<?php

/**
 * FiloBlu Controller for Catalor Search
 *
 * @category    FiloBlu
 * @package     FiloBlu_AjaxLayeredNavigation
 * @author      Vincenzo Assenza
 *
 */

namespace FiloBlu\AjaxLayeredNavigation\Controller\CatalogSearch\Result;

use Magento\CatalogSearch\Controller\Result;
use Magento\Catalog\Model\Session;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\ResourceConnection;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Search\Model\QueryFactory;
use Magento\Catalog\Model\Layer\Resolver;
use Magento\Framework\View\Result\PageFactory;
use Magento\Framework\Controller\Result\JsonFactory;

class Index extends \Magento\CatalogSearch\Controller\Result\Index {
    
    protected $resultJsonFactory;
    protected $resultPageFactory;
    protected $_catalogLayer;
    
    /**
     * Catalog session
     *
     * @var Session
     */
    protected $_catalogSession;

    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var QueryFactory
     */
    private $_queryFactory;

    /**
     * Catalog Layer Resolver
     *
     * @var Resolver
     */
    private $layerResolver;

    /**
     * @param Context $context
     * @param Session $catalogSession
     * @param StoreManagerInterface $storeManager
     * @param QueryFactory $queryFactory
     * @param Resolver $layerResolver
     */
    public function __construct(
        Context $context,
        Session $catalogSession,
        StoreManagerInterface $storeManager,
        QueryFactory $queryFactory,
        Resolver $layerResolver,
        PageFactory $resultPageFactory,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
    ) {
        $this->_storeManager = $storeManager;
        $this->_catalogSession = $catalogSession;
        $this->_queryFactory = $queryFactory;
        $this->layerResolver = $layerResolver;
        parent::__construct($context,$catalogSession,$storeManager,$queryFactory,$layerResolver);
        $this->resultPageFactory = $resultPageFactory;
        $this->resultJsonFactory = $resultJsonFactory;
    }
    
    /**
     * Execute controller
     */
    public function execute() {
        if ($this->getRequest()->isXmlHttpRequest() &&
                $this->_objectManager->get('Magento\Framework\App\Config\ScopeConfigInterface')->getValue('filoblu_ajaxlayerednavigation/general/enabled', \Magento\Store\Model\ScopeInterface::SCOPE_STORE) == 1) {
            return $this->_viewAjax();
        } else {
            return parent::execute();
        }
    }

    private function _viewAjax() {
        $this->layerResolver->create(Resolver::CATALOG_LAYER_SEARCH);
        $query = $this->_queryFactory->get();

        $query->setStoreId($this->_storeManager->getStore()->getId());

        if ($query->getQueryText() != '') {
            if ($this->_objectManager->get('Magento\CatalogSearch\Helper\Data')->isMinQueryLength()) {
                $query->setId(0)->setIsActive(1)->setIsProcessed(1);
            } else {
                $query->saveIncrementalPopularity();

                if ($query->getRedirect()) {
                    $this->getResponse()->setRedirect($query->getRedirect());
                    return;
                }
            }

            $this->_objectManager->get('Magento\CatalogSearch\Helper\Data')->checkNotes();

            //ajax response
            return $this->_jsonResponse();
        } else {
            $this->_redirectReferer();
        }
    }

    private function _jsonResponse() {
        
        $response = array();

        $blockList = unserialize(
                $this->_objectManager->get('Magento\Framework\App\Config\ScopeConfigInterface')->getValue('filoblu_ajaxlayerednavigation/general/block_list', \Magento\Store\Model\ScopeInterface::SCOPE_STORE)
        );
        
        $page = $this->resultPageFactory->create();

        foreach ($blockList as $singleBlock) {
            $singleXmlName = trim($singleBlock["xml_name"]);
            $singleDomName = $singleBlock["dom_name"];
            $block = $page->getLayout()->getBlock($singleXmlName);
            if ($block) {
                $response[$singleDomName] = $block->toHtml();
            }
        }

        $resultJson = $this->resultJsonFactory->create();
        return $resultJson->setData($response);
    }
}
