<?php

/**
 * FiloBlu Controller for Category View
 *
 * @category    FiloBlu
 * @package     FiloBlu_AjaxLayeredNavigation
 * @author      Vincenzo Assenza
 *
 */

namespace FiloBlu\AjaxLayeredNavigation\Controller\Category;

use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Model\Layer\Resolver;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\View\Result\PageFactory;
use Magento\Framework\Controller\Result\JsonFactory;

/**
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class View extends \Magento\Catalog\Controller\Category\View {

    public $layerResolver;
    protected $resultJsonFactory;

    public function __construct
    (
        \Magento\Framework\App\Action\Context $context,
        \Magento\Catalog\Model\Design $catalogDesign,
        \Magento\Catalog\Model\Session $catalogSession,
        \Magento\Framework\Registry $coreRegistry,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\CatalogUrlRewrite\Model\CategoryUrlPathGenerator $categoryUrlPathGenerator,
        PageFactory $resultPageFactory,
        \Magento\Framework\Controller\Result\ForwardFactory $resultForwardFactory,
        Resolver $layerResolver,
        CategoryRepositoryInterface $categoryRepository,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
    ) {

        $this->_storeManager = $storeManager;
        $this->_catalogDesign = $catalogDesign;
        $this->_catalogSession = $catalogSession;
        $this->_coreRegistry = $coreRegistry;
        $this->categoryUrlPathGenerator = $categoryUrlPathGenerator;
        $this->resultPageFactory = $resultPageFactory;
        $this->resultForwardFactory = $resultForwardFactory;
        $this->layerResolver = $layerResolver;
        $this->categoryRepository = $categoryRepository;

        parent::__construct($context, $catalogDesign, $catalogSession, $coreRegistry, $storeManager, $categoryUrlPathGenerator, $resultPageFactory, $resultForwardFactory, $layerResolver, $categoryRepository);

        $this->resultJsonFactory = $resultJsonFactory;
    }

    /**
     * Product list page
     *
     * @return \Magento\Backend\Model\View\Result\Page
     */
    public function execute() {

        if ($this->getRequest()->isXmlHttpRequest() &&
                $this->_objectManager->get('Magento\Framework\App\Config\ScopeConfigInterface')->getValue('filoblu_ajaxlayerednavigation/general/enabled', \Magento\Store\Model\ScopeInterface::SCOPE_STORE) == 1) {
            return $this->_viewAjax();
        } else {
            return parent::execute();
        }
    }

    private function _viewAjax() {

        $category = $this->_initCategory();

        if ($category) {

            $this->layerResolver->create(Resolver::CATALOG_LAYER_CATEGORY);
            $settings = $this->_catalogDesign->getDesignSettings($category);

            // apply custom design
            if ($settings->getCustomDesign()) {
                $this->_catalogDesign->applyCustomDesign($settings->getCustomDesign());
            }

            $this->_catalogSession->setLastViewedCategoryId($category->getId());

            $page = $this->resultPageFactory->create();
            // apply custom layout (page) template once the blocks are generated
            if ($settings->getPageLayout()) {
                $page->getConfig()->setPageLayout($settings->getPageLayout());
            }

            $hasChildren = $category->hasChildren();
            if ($category->getIsAnchor()) {
                $type = $hasChildren ? 'layered' : 'layered_without_children';
            } else {
                $type = $hasChildren ? 'default' : 'default_without_children';
            }

            if (!$hasChildren) {
                // Two levels removed from parent.  Need to add default page type.
                $parentType = strtok($type, '_');
                $page->addPageLayoutHandles(['type' => $parentType]);
            }
            $page->addPageLayoutHandles(['type' => $type, 'id' => $category->getId()]);

            // apply custom layout update once layout is loaded
            $layoutUpdates = $settings->getLayoutUpdates();
            if ($layoutUpdates && is_array($layoutUpdates)) {
                foreach ($layoutUpdates as $layoutUpdate) {
                    $page->addUpdate($layoutUpdate);
                    $page->addPageLayoutHandles(['layout_update' => md5($layoutUpdate)]);
                }
            }

            $page->getConfig()->addBodyClass('page-products')
                    ->addBodyClass('categorypath-' . $this->categoryUrlPathGenerator->getUrlPath($category))
                    ->addBodyClass('category-' . $category->getUrlKey());

            //ajax response
            return $this->_jsonResponse($page);
        } elseif (!$this->getResponse()->isRedirect()) {
            return $this->resultForwardFactory->create()->forward('noroute');
        }
    }

    private function _jsonResponse($page) {
        $response = array();

        $blockList = unserialize(
                $this->_objectManager->get('Magento\Framework\App\Config\ScopeConfigInterface')->getValue('filoblu_ajaxlayerednavigation/general/block_list', \Magento\Store\Model\ScopeInterface::SCOPE_STORE)
        );

        foreach ($blockList as $singleBlock) {
            $singleXmlName = trim($singleBlock["xml_name"]);
            $singleDomName = $singleBlock["dom_name"];
            $block = $page->getLayout()->getBlock($singleXmlName);
            if ($block) {
                $response[$singleDomName] = $block->toHtml();
            }
        }

        $resultJson = $this->resultJsonFactory->create();
        return $resultJson->setData($response);
    }

}
